@@ -21,7 +21,6 @@
 
 #include "config.h"
 
-#include <ctype.h>
 #include <stdio.h>
 #include <string.h>
 #include <limits.h>
@@ -163,7 +162,7 @@ pango_engine_pair_get_engine (PangoEnginePair *pair)
 	      return NULL;
 	    }
 	  
-	  g_module_symbol (module, "script_engine_load", (gpointer)&load);
+	  g_module_symbol (module, "script_engine_load", (gpointer *) &load);
 	  if (!load)
 	    {
 	      fprintf(stderr, "cannot retrieve script_engine_load from %s: %s\n",
@@ -246,8 +245,8 @@ process_module_file (FILE *module_file)
 	      pair->load_info = g_strdup (tmp_buf->str);
 #if defined(G_OS_WIN32) && defined(LIBDIR)
 	      if (strncmp (pair->load_info,
-			   LIBDIR "/pango/modules/",
-			   strlen (LIBDIR "/pango/modules/")) == 0)
+			   LIBDIR "/pango/" MODULE_VERSION "/modules/",
+			   strlen (LIBDIR "/pango/" MODULE_VERSION "/modules/")) == 0)
 		{
 		  /* This is an entry put there by make install on the
 		   * packager's system. On Windows a prebuilt Pango
@@ -259,8 +258,8 @@ process_module_file (FILE *module_file)
 		  gchar *tem = pair->load_info;
 		  pair->load_info =
 		    g_strconcat (pango_get_lib_subdirectory (),
-				 "\\modules\\",
-				 tem + strlen (LIBDIR "/pango/modules/"),
+				 "\\" MODULE_VERSION "\\modules\\",
+				 tem + strlen (LIBDIR "/pango/" MODULE_VERSION "/modules/"),
 				 NULL);
 		  g_free (tem);
 		}
@@ -501,7 +500,7 @@ build_map (PangoMapInfo *info)
   init_modules();
 
   info->map = map = g_new (PangoMap, 1);
-  map->n_submaps = 0;
+  map->n_submaps = 256;
   for (i=0; i<256; i++)
     {
       map->submaps[i].is_leaf = TRUE;
@@ -520,9 +519,9 @@ build_map (PangoMapInfo *info)
  * @wc:  an ISO-10646 codepoint
  * 
  * Returns the entry in the map for a given codepoint. The entry
- * contains information about engine that should be used for
+ * contains information about the engine that should be used for
  * the codepoint and also whether the engine matches the language
- * tag for the map was created exactly or just approximately.
+ * tag for which the map was created exactly or just approximately.
  * 
  * Return value: the #PangoMapEntry for the codepoint. This value
  *   is owned by the #PangoMap and should not be freed.
@@ -531,8 +530,18 @@ PangoMapEntry *
 pango_map_get_entry (PangoMap   *map,
 		     guint32     wc)
 {
-  PangoSubmap *submap = &map->submaps[wc / 256];
-  return submap->is_leaf ? &submap->d.entry : &submap->d.leaves[wc % 256];
+  int i = wc / 256;
+
+  if (i < map->n_submaps)
+    {
+      PangoSubmap *submap = &map->submaps[i];
+      return submap->is_leaf ? &submap->d.entry : &submap->d.leaves[wc % 256];
+    }
+  else
+    {
+      static PangoMapEntry default_entry = { NULL, FALSE };
+      return &default_entry;
+    }
 }
 
 /**
@@ -550,11 +559,18 @@ PangoEngine *
 pango_map_get_engine (PangoMap *map,
 		      guint32   wc)
 {
-  PangoSubmap *submap = &map->submaps[wc / 256];
-  PangoMapEntry *entry = submap->is_leaf ? &submap->d.entry : &submap->d.leaves[wc % 256];
+  int i = wc / 256;
 
-  if (entry->info)
-    return pango_engine_pair_get_engine ((PangoEnginePair *)entry->info);
+  if (i < map->n_submaps)
+    {
+      PangoSubmap *submap = &map->submaps[i];
+      PangoMapEntry *entry = submap->is_leaf ? &submap->d.entry : &submap->d.leaves[wc % 256];
+      
+      if (entry->info)
+	return pango_engine_pair_get_engine ((PangoEnginePair *)entry->info);
+      else
+	return NULL;
+    }
   else
     return NULL;
 }
